

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>


int GetCreationTime(const char *path, time_t* t);
int SetCreationTime(const char *path, time_t t);

int set_time(const char *fname, const struct stat *sbuf);
int set_ctime(const char *fname, const struct stat *sbuf);


int make_writable(const char *fname, const struct stat *sbuf);
int restore_writable(const char *fname, const struct stat *sbuf);

int has_locks(const struct stat *sbuf);
int remove_locks(const char *fname, const struct stat *sbuf);

int has_xattr(const char *fname);
int strip_xattr(const char *fname);

int split_xattr(const char *fname, const struct stat *sbuf, 
                  const char *cname, int crtime);

int join_xattr(const char *fname, const struct stat *sbuf, const char *cname);

int skip_xattr(const char *cname);


