

/* usage: join1_xattr fname cname
 * 
 * undoes split1_xattr fname, reading xattr container for fname from stdin
 * 
 * Returns non-zero if errors detected, and 0 otherwise.
 *
 */

#include "util.h"
#include "xattr_util.h"

void usage()
{
   fprintf(stderr, "usage: join1_xattr fname\n");
}

int main(int argc, char **argv)
{
   char *fname;
   struct stat sbuf;

   if (argc != 2) {
      usage();
      return -1;
   }

   fname = argv[1];

   if (lstat(fname, &sbuf))
      return -1;

   return join_xattr(fname, &sbuf, "");
}


