
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <sys/time.h>


#define MAXLEN (2048)


#define DBL_PREFIX "@_"
#define DBL_PREFIX_LEN (2)

#define S_ISREG_OR_LNK(m) (S_ISREG(m) || S_ISLNK(m))

int is_prefix(const char *pat, const char *txt);


void collect_names(const char *fname);
int lookup_name(const char *s);

void overflow(void);



